@@ -4,18 +4,20 @@
 """
 
 import pickle  # nosec
+import re
 import signal
 from multiprocessing import Pool, cpu_count
 from multiprocessing.pool import AsyncResult
 from pathlib import Path
-from typing import Dict, List, NamedTuple, Optional, TypeVar, Tuple, Union
+from typing import Dict, List, NamedTuple, Optional, Tuple, Union
 
+import matplotlib.pyplot as plt
 import numpy as np
-import numpy.typing as npt
 import pandas as pd
 import plotly.graph_objects as go
 import pysam
 
+from numpy.typing import ArrayLike
 from scipy import stats
 from tqdm.auto import tqdm
 
@@ -23,8 +25,6 @@ from . import quma
 from . import visualization as viz
 from .config import Config
 
-GPD = TypeVar("GPD", bound="GenomicPositionData")
-
 # Initialized multiprocessing limits
 NUM_THREADS = cpu_count() - 1
 
@@ -37,8 +37,8 @@ class AD_stats(NamedTuple):
     """Helper class for NamedTuple results from anderson_darling_test"""
 
     sig: bool
-    stat: npt.ArrayLike
-    crits: List[npt.ArrayLike]
+    stat: ArrayLike
+    crits: List[ArrayLike]
 
 
 class BamOutput:
@@ -75,8 +75,8 @@ class BamOutput:
         samm: pysam.AlignmentFile = pysam.AlignmentFile(str(sams), "rb")
         itern = samm.fetch(
             self._config.chromosome,
-            int(self._config.promoter_start),
-            int(self._config.promoter_end),
+            self._config.promoter_start,
+            self._config.promoter_end,
         )
 
         position: List[str] = []
@@ -370,7 +370,10 @@ class GenomicPositionData:
         for name, bam_result in tqdm(
             self._bam_output.items(), desc="Process methylation"
         ):
-            cell_line_name: str = Path(name).name.split("_")[1]
+            # cell_line_name: str = Path(name).name.split("_")[1]
+            cell_line_name: str = re.match(
+                self.config.fname_pattern, Path(name).name
+            ).group(1)
 
             read_files: List[str] = [each for each in bam_result.values.values()]
             genomic_files: List[str] = [
@@ -411,14 +414,14 @@ class GenomicPositionData:
             pickle.dump(self, output_file)
 
     @staticmethod
-    def from_pickle(filename: str) -> GPD:
+    def from_pickle(filename: str) -> "GenomicPositionData":
         """Read pickled GenomicPositionData back to an object.
 
         Args:
             filename (str): filename to read pickle
 
         Returns:
-            GPD: GenomicPositionData object
+            GenomicPositionData: GenomicPositionData object
         """
 
         with open(filename, "rb") as input_file:
@@ -442,7 +445,7 @@ class GenomicPositionData:
                 values_list: List[float] = self._return_read_values(pos, key)
                 if values_list:
                     pos_means: float = float(np.mean(values_list))
-                else:  # No data or data doesn't meet minimums for analysis
+                else:  # pragma: no cover
                     pos_means = np.nan
 
                 working_df.at[key, pos] = pos_means
@@ -468,7 +471,7 @@ class GenomicPositionData:
 
                 if values_list:
                     pos_modes: float = stats.mode(values_list)[0][0]
-                else:  # No data or data doesn't meet minimums for analysis
+                else:  # pragma: no cover
                     pos_modes = np.nan
 
                 working_df.at[key, pos] = pos_modes
@@ -492,7 +495,7 @@ class GenomicPositionData:
                 values_list: List[float] = self._return_read_values(pos, key)
                 if values_list:
                     data_for_df: Union[List[float], float] = values_list
-                else:  # No data or data doesn't meet minimums for analysis
+                else:  # pragma: no cover
                     data_for_df = np.nan
 
                 working_df.at[key, pos] = data_for_df
@@ -604,34 +607,64 @@ class GenomicPositionData:
             index=True,
         )
 
-    def histogram(self, cell_line: str, position: str) -> None:
+    def histogram(
+        self, cell_line: str, position: str, backend: Optional[str] = None
+    ) -> None:
         """Display a graph figure showing fractional methylation in
         a given cell line at a given site.
 
         Args:
             cell_line (str): name of cell line
             position (str): genomic position
+            backend (Optional[str]): plotting backend to override default
+
+        Raises:
+            ValueError: invalid plotting backend
         """
         data = self.individual_data
+        if not backend:
+            backend = self.config.viz_backend
 
-        fig: go.Figure = viz._create_histogram(data, cell_line, position)
-        fig.show()
+        if backend == "plotly":
+            fig: go.Figure = viz._create_histogram(data, cell_line, position, backend)
+            fig.show()
+            return
+        if backend == "matplotlib":
+            _: plt.Figure = viz._create_histogram(data, cell_line, position, backend)
+            plt.show()
+            return
+        raise ValueError("Invalid plotting backend")
 
-    def reads_graph(self, cell_lines: Optional[List[str]] = None) -> None:
+    def reads_graph(
+        self, cell_lines: Optional[List[str]] = None, backend: Optional[str] = None
+    ) -> None:
         """Display a graph figure showing methylation of reads across cell lines.
 
         Args:
             cell_lines (Optional[List[str]]): set of cell lines to analyze,
-            defaults to all cell lines.
+                                            defaults to all cell lines.
+            backend (Optional[str]): plotting backend to override default
+
+        Raises:
+            ValueError: invalid plotting backend
         """
 
         data = self.individual_data
+        if not backend:
+            backend = self.config.viz_backend
 
         if cell_lines:
             data = data[data.index.isin(cell_lines)]
 
-        fig: go.Figure = viz._make_stacked_fig(data)
-        fig.show()
+        if backend == "plotly":
+            fig: go.Figure = viz._make_stacked_fig(data, backend)
+            fig.show()
+            return
+        if backend == "matplotlib":
+            _: plt.Figure = viz._make_stacked_fig(data, backend)
+            plt.show()
+            return
+        raise ValueError("Invalid plotting backend")
 
     def heatmap(
         self,
@@ -640,6 +673,7 @@ class GenomicPositionData:
         height: int = 2000,
         cell_lines: Optional[List[str]] = None,
         data_type: str = "means",
+        backend: Optional[str] = None,
     ) -> None:
         """Display a graph figure showing heatmap of mean methylation across
         cell lines.
@@ -651,9 +685,11 @@ class GenomicPositionData:
             cell_lines (Optional[List[str]]): set of cell lines to analyze,
             defaults to all cell lines.
             data_type (str): type of data to plot. Can to 'means', 'modes', or 'diffs'
+            backend (Optional[str]): plotting backend to override default
 
         Raises:
             ValueError: invalid data type
+            ValueError: invalid plotting backend
         """
 
         title_type: str
@@ -673,25 +709,55 @@ class GenomicPositionData:
         if cell_lines:
             data = data[data.index.isin(cell_lines)]
 
-        fig: go.Figure = viz._create_heatmap(
-            data, min_values, width, height, title_type
-        )
-        fig.show()
+        if not backend:
+            backend = self.config.viz_backend
+
+        if backend == "plotly":
+            fig: go.Figure = viz._create_heatmap(
+                data, min_values, width, height, title_type, backend
+            )
+            fig.show()
+            return
+
+        if backend == "matplotlib":
+            _: plt.Figure = viz._create_heatmap(
+                data, min_values, width, height, title_type, backend
+            )
+            plt.show()
+            return
+
+        raise ValueError("Invalid plotting backend")
 
     def sig_methylation_differences(
-        self, cell_lines: Optional[List[str]] = None
+        self,
+        cell_lines: Optional[List[str]] = None,
+        backend: Optional[str] = None,
     ) -> None:
         """Display a graph figure showing a bar chart of significantly different
         mean / mode methylation across all or a subset of cell lines.
 
         Args:
             cell_lines (Optional[List[str]]): set of cell lines to analyze,
-            defaults to all cell lines.
+                                            defaults to all cell lines.
+            backend (Optional[str]): plotting backend to override default
+
+        Raises:
+            ValueError: invalid plotting backend
         """
         data = self.summarize_allelic_data(cell_lines)
 
-        fig: go.Figure = viz._create_methylation_diffs_bar_graph(data)
-        fig.show()
+        if not backend:
+            backend = self.config.viz_backend
+
+        if backend == "plotly":
+            fig: go.Figure = viz._create_methylation_diffs_bar_graph(data, backend)
+            fig.show()
+            return
+        if backend == "matplotlib":
+            _: plt.Figure = viz._create_methylation_diffs_bar_graph(data, backend)
+            plt.show()
+            return
+        raise ValueError("Invalid plotting backend")
 
     def generate_ad_stats(self) -> pd.DataFrame:
         """Generate Anderson-Darling normality statistics for an individual data df.
@@ -723,7 +789,7 @@ class GenomicPositionData:
             data = self.individual_data
 
         np.seterr(divide="ignore", invalid="ignore")  # ignore divide-by-zero errors
-        sig_dict: Dict[str, List[npt.ArrayLike]] = {
+        sig_dict: Dict[str, List[ArrayLike]] = {
             "cellLine": [],
             "position": [],
             "ad_stat": [],
@@ -759,8 +825,8 @@ class GenomicPositionData:
         """
 
         if np.all(pd.notnull(raw_list)):
-            stat: npt.ArrayLike
-            crits: List[npt.ArrayLike]
+            stat: ArrayLike
+            crits: List[ArrayLike]
             stat, crits, _ = stats.anderson(raw_list)
             is_sig: bool = bool(stat > crits[4])  # type: ignore
             return AD_stats(is_sig, stat, crits)
@@ -776,10 +842,14 @@ class GenomicPositionData:
 def configure(
     base_path: str,
     prom_file: str,
-    prom_start: str,
-    prom_end: str,
+    prom_start: int,
+    prom_end: int,
     chrom: str,
     offset: int,
+    test_dir: Optional[str] = None,
+    fname_pattern: Optional[str] = None,
+    viz_backend: Optional[str] = None,
+    results_dir: Optional[str] = None,
 ) -> Config:
     """Helper method to set up all our environmental variables, such as for testing.
 
@@ -787,25 +857,42 @@ def configure(
         base_path (str): directory where all processing will occur, put .bam files
                          in "test" sub-directory in this folder
         prom_file (str): filename of genmic sequence of promoter region of interest
-        prom_start (str): start position to analyze in promoter region
-        prom_end (str): final position to analyze in promoter region
+        prom_start (int): start position to analyze in promoter region
+        prom_end (int): final position to analyze in promoter region
         chrom (str): chromosome promoter is located on
         offset (int): genomic position of promoter to offset reads
+        test_dir(Optional[str]): name of test directory where bam files are located
+        fname_pattern(Optional[str]): regex pattern for processing filenames
+        viz_backend (Optional[str]): which plotting backend to use
+        results_dir (Optional[str]): name of results directory
 
     Returns:
         Config: configuration dataclass instance.
     """
 
+    if not test_dir:
+        test_dir = "test"
+
+    if not fname_pattern:
+        fname_pattern = r"^[a-zA-Z]+_([a-zA-Z0-9]+)_.+bam$"
+
+    if not viz_backend:
+        viz_backend = "plotly"
+
+    if not results_dir:
+        results_dir = "results"
+
     config = Config(
         base_directory=Path(base_path),
         promoter_file=Path(base_path) / prom_file,
-        results_directory=Path(base_path) / "results",
-        bam_directory=Path(base_path) / "bam_output",
-        analysis_directory=Path(base_path) / "test",
+        results_directory=Path(base_path) / results_dir,
+        analysis_directory=Path(base_path) / test_dir,
         promoter_start=prom_start,
         promoter_end=prom_end,
         chromosome=chrom,
         offset=offset,
+        viz_backend=viz_backend,
+        fname_pattern=fname_pattern,
     )
 
     return config
@@ -821,3 +908,16 @@ def make_list_of_bam_files(config: Config) -> List[str]:
         list[str]: list of files
     """
     return [f.name for f in config.analysis_directory.iterdir() if f.suffix == ".bam"]
+
+
+def pyllelic(config: Config, files_set: List[str]) -> GenomicPositionData:
+    """Wrapper to call pyllelic routines.
+
+    Args:
+        config (Config): pyllelic config object.
+        files_set (List[str]): list of bam files to analyze.
+
+    Returns:
+        GenomicPositionData: GenomicPositionData pyllelic object.
+    """
+    return GenomicPositionData(config=config, files_set=files_set)
