@@ -8,7 +8,7 @@ import unittest.mock as mock
 
 # import os
 
-from inputs import (
+from .inputs import (
     EXPECTED_INTERMEDIATE_DIFFS,
     SAMPLE_BAM,
     SAMPLE_BAI,
@@ -40,7 +40,7 @@ def set_up_genomic_position_data(tmp_path_factory):
     tmp_path = tmp_path_factory.mktemp("data")
     p, _ = setup_bam_files(tmp_path)
     config = setup_config(p)
-    INPUT_BAM_LIST = ["fh_test.bam"]
+    INPUT_BAM_LIST = ["fh_test_tissue.bam"]
     return (
         p,
         pyllelic.GenomicPositionData(config=config, files_set=INPUT_BAM_LIST),
@@ -70,10 +70,10 @@ class MockPoolApplyResult:
 def setup_bam_files(tmp_path):
     d = tmp_path / "test"
     d.mkdir()
-    fn_bam = "fh_test.bam"
+    fn_bam = "fh_test_tissue.bam"
     filepath_bam = d / fn_bam
     filepath_bam.write_bytes(base64.decodebytes(SAMPLE_BAM))
-    fn_bai = "fh_test.bam.bai"
+    fn_bai = "fh_test_tissue.bam.bai"
     filepath_bai = d / fn_bai
     filepath_bai.write_bytes(base64.decodebytes(SAMPLE_BAI))
     return tmp_path, filepath_bam
@@ -83,10 +83,10 @@ def setup_config(my_path):
     d = my_path
     prom_file = d / "test.txt"
     prom_file.write_text(TEST_PROM_FILE)
-    TEST_START = "1293200"
-    TEST_END = "1296000"
+    TEST_START = 1293200
+    TEST_END = 1296000
     TEST_CHR = "5"
-    TEST_OFFSET = 1298163
+    TEST_OFFSET = 1293000
     config = pyllelic.configure(
         base_path=my_path,
         prom_file=prom_file,
@@ -99,6 +99,16 @@ def setup_config(my_path):
     return config
 
 
+# Repeated test parameters
+TEST_QUMA_RESULT = (
+    "genome\t0\tATCGTAGTCGA\t2\t2,8\n"
+    + "1\tquery1\tATCGTAGTCGA\tATCGTAGTCGA\tATCGTAGTCGA\t"
+    + "11\t0\t100.0\t0\t2\t0\t2\t100.0\t11\t1\t1\n"
+    + "2\tquery2\tATCGATAGCATT\tATCGATAGCATT\tATCG-TAGTCGA\t"
+    + "12\t5\t58.3\t1\t1\t0\t1\t100.0\t1A\t1\t1\n"
+)
+
+
 # Tests
 def test_configure():
     """Test setting environment variables with mock object."""
@@ -130,10 +140,10 @@ def test_make_list_of_bam_files(tmp_path):
     TEST_LIST = [
         Path("bad1.txt"),
         Path("bad2.bai"),
-        Path("good1.bam"),
-        Path("good2.bam"),
+        Path("fh_good1_tissue.bam"),
+        Path("fh_good2_tissue.bam"),
     ]
-    EXPECTED = ["good1.bam", "good2.bam"]
+    EXPECTED = ["fh_good1_tissue.bam", "fh_good2_tissue.bam"]
     with mock.patch.object(pyllelic.Path, "iterdir") as mock_iterdir:
         mock_iterdir.return_value = TEST_LIST
         actual = pyllelic.make_list_of_bam_files(config)
@@ -141,6 +151,22 @@ def test_make_list_of_bam_files(tmp_path):
     assert EXPECTED == actual
 
 
+def test_pyllelic(tmp_path_factory):
+    tmp_path = tmp_path_factory.mktemp("data")
+    p, _ = setup_bam_files(tmp_path)
+    config = setup_config(p)
+    INPUT_BAM_LIST = ["fh_test_tissue.bam"]
+    genomic_position_data = pyllelic.pyllelic(config=config, files_set=INPUT_BAM_LIST)
+    positions = []
+    for each in EXPECTED_BAM_OUTPUT_POSITIONS:
+        positions.append(each)
+
+    EXPECTED_POSITIONS = sorted(set(positions))
+
+    assert genomic_position_data.positions == EXPECTED_POSITIONS
+    assert genomic_position_data.cell_types == [str(p / "test" / "fh_test_tissue.bam")]
+
+
 # Tests of main classes
 
 
@@ -162,7 +188,7 @@ class Test_BamOutput:
     def test_init(self, tmp_path, set_up_bam_output):
         bam_output = set_up_bam_output
 
-        assert bam_output.name == str(tmp_path / "test" / "fh_test.bam")
+        assert bam_output.name == str(tmp_path / "test" / "fh_test_tissue.bam")
         assert bam_output.values == EXPECTED_BAM_OUTPUT_VALUES
         assert bam_output.positions == EXPECTED_BAM_OUTPUT_POSITIONS
         assert bam_output.genome_values == EXPECTED_BAM_OUTPUT_GENOME_VALUES
@@ -174,18 +200,9 @@ class Test_BamOutput:
 
     def test_write_bam_output(self, set_up_bam_output):
         bam_output = set_up_bam_output
-        # print(f"Initial bam values:\n{bam_output.values}")
         bam_output._write_bam_output(bam_output.positions, EXPECTED_STACKED_BAM)
-        # print(f"Revised bam values:\n{bam_output.values}")
         assert bam_output.values == EXPECTED_WRITE_DF_OUTPUT
 
-    # @mock.patch("pyllelic.pyllelic.pysam")
-    # def test_pysam_index(self, mock_pysam, set_up_bam_output):
-    #     bam_output = set_up_bam_output
-    #     TEST_PATH = Path().cwd()
-    #     bam_output._pysam_index(TEST_PATH)
-    #     mock_pysam.index.assert_called_once_with(os.fspath(TEST_PATH))
-
     def test__genome_range(self, set_up_bam_output):
         bam_output = set_up_bam_output
         gen_str = "ATGCATGCATGCATGCATGCATGCATGCATGCATGCATGCATGCATGC"
@@ -243,17 +260,18 @@ class Test_QumaOutput:
 
     def test_process_raw_quma(self, set_up_quma_output):
         quma_output = set_up_quma_output
-        TEST_QUMA_RESULT = (
-            "genome\t0\tATCGTAGTCGA\t2\t2,8\n"
-            + "1\tquery1\tATCGTAGTCGA\tATCGTAGTCGA\tATCGTAGTCGA\t"
-            + "11\t0\t100.0\t0\t2\t0\t2\t100.0\t11\t1\t1\n"
-            + "2\tquery2\tATCGATAGCATT\tATCGATAGCATT\tATCG-TAGTCGA\t"
-            + "12\t5\t58.3\t1\t1\t0\t1\t100.0\t1A\t1\t1\n"
-        )
+
         EXPECTED = ["1A", "11"]
         actual = quma_output._process_raw_quma(TEST_QUMA_RESULT)
         assert EXPECTED == actual
 
+    def test_process_raw_quma_below_min_alignment(self, set_up_quma_output):
+        quma_output = set_up_quma_output
+        TEST_QUMA_RESULT_FAIL = TEST_QUMA_RESULT[:157] + "4" + TEST_QUMA_RESULT[158:]
+        EXPECTED = ["FAIL", "11"]
+        actual = quma_output._process_raw_quma(TEST_QUMA_RESULT_FAIL)
+        assert EXPECTED == actual
+
     def test__pool_processing(self, set_up_quma_output):
         quma_output = set_up_quma_output
 
@@ -316,7 +334,9 @@ class Test_GenomicPositionData:
         EXPECTED_POSITIONS = sorted(set(positions))
 
         assert genomic_position_data.positions == EXPECTED_POSITIONS
-        assert genomic_position_data.cell_types == [str(p / "test" / "fh_test.bam")]
+        assert genomic_position_data.cell_types == [
+            str(p / "test" / "fh_test_tissue.bam")
+        ]
 
     # def test_save(self, set_up_genomic_position_data, mocker):
     #     _, genomic_position_data = set_up_genomic_position_data
@@ -334,68 +354,71 @@ class Test_GenomicPositionData:
         np.array_equal(genomic_position_data.means.values, expected.values)
         # pd.testing.assert_frame_equal(genomic_position_data.means, expected)
 
-    def test__create_histogram(self, mocker, set_up_genomic_position_data):
-        _, genomic_position_data = set_up_genomic_position_data
-        mocked_go = mocker.patch("pyllelic.pyllelic.go")
-        TEST_CELL_LINE = "TEST1"
-        TEST_POSITION = "1"
-        intermediate = EXPECTED_INTERMEDIATE_INDIVIDUAL_DATA
-        TEST_DATA = intermediate.astype("object")
-
-        _ = genomic_position_data._create_histogram(
-            TEST_DATA, TEST_CELL_LINE, TEST_POSITION
-        )
-
-        mocked_go.Figure.assert_called_once()
-        mocked_go.Histogram.assert_called_once_with(
-            x=TEST_DATA.loc[TEST_CELL_LINE, TEST_POSITION],
-            xbins=dict(
-                start=-0.1,
-                end=1.1,
-                size=0.2,
-            ),
-        )
-
     def test_histogram(self, set_up_genomic_position_data, mocker):
         _, genomic_position_data = set_up_genomic_position_data
-        mocked_go = mocker.patch("pyllelic.pyllelic.go")
+        mocked_go = mocker.patch("pyllelic.visualization.go")
         TEST_POSITION = genomic_position_data.positions[0]
         TEST_CELL_LINE = genomic_position_data.means.index[0]
 
         genomic_position_data.histogram(TEST_CELL_LINE, TEST_POSITION)
+        genomic_position_data.histogram(TEST_CELL_LINE, TEST_POSITION, backend="plotly")
 
-        mocked_go.Figure.assert_called_once()
-        mocked_go.Histogram.assert_called_once()
+        mocked_go.Figure.assert_called()
+        mocked_go.Histogram.assert_called()
 
-    def test__create_heatmap(self, mocker, set_up_genomic_position_data):
+    def test_histogram_mpl(self, set_up_genomic_position_data, mocker):
         _, genomic_position_data = set_up_genomic_position_data
-        mocked_go = mocker.patch("pyllelic.pyllelic.go")
-        intermediate = EXPECTED_INTERMEDIATE_INDIVIDUAL_DATA
-        TEST_DATA = intermediate.astype("object")
+        mocked_mpl = mocker.patch("pyllelic.visualization.sns")
+        TEST_POSITION = genomic_position_data.positions[0]
+        TEST_CELL_LINE = genomic_position_data.means.index[0]
 
-        _ = genomic_position_data._create_heatmap(
-            TEST_DATA,
-            min_values=1,
-            height=600,
-            width=600,
-            title_type="means",
-        )
+        genomic_position_data.config.viz_backend = "matplotlib"
+        genomic_position_data.histogram(TEST_CELL_LINE, TEST_POSITION)
 
-        mocked_go.Figure.assert_called_once()
-        mocked_go.Heatmap.assert_called_once()
+        mocked_mpl.histplot.assert_called_once()
+        genomic_position_data.config.viz_backend = "plotly"
+
+    def test_histogram_error(self, set_up_genomic_position_data):
+        _, genomic_position_data = set_up_genomic_position_data
+        TEST_POSITION = genomic_position_data.positions[0]
+        TEST_CELL_LINE = genomic_position_data.means.index[0]
+
+        with pytest.raises(ValueError):
+            genomic_position_data.config.viz_backend = "other"
+            genomic_position_data.histogram(TEST_CELL_LINE, TEST_POSITION)
+        genomic_position_data.config.viz_backend = "plotly"
 
     def test_heatmap(self, set_up_genomic_position_data, mocker):
         _, genomic_position_data = set_up_genomic_position_data
-        mocked_go = mocker.patch("pyllelic.pyllelic.go")
+        mocked_go = mocker.patch("pyllelic.visualization.go")
 
         genomic_position_data.heatmap(min_values=1)
+        genomic_position_data.heatmap(min_values=1, backend="plotly")
 
-        mocked_go.Figure.assert_called_once()
-        mocked_go.Heatmap.assert_called_once()
+        mocked_go.Figure.assert_called()
+        mocked_go.Heatmap.assert_called()
+
+    def test_heatmap_mpl(self, set_up_genomic_position_data, mocker):
+        _, genomic_position_data = set_up_genomic_position_data
+        mocked_mpl = mocker.patch("pyllelic.visualization.sns")
+
+        genomic_position_data.config.viz_backend = "matplotlib"
+        genomic_position_data.heatmap(min_values=1)
+
+        mocked_mpl.heatmap.assert_called_once()
+        genomic_position_data.config.viz_backend = "plotly"
+
+    def test_heatmap_error(self, set_up_genomic_position_data):
+        _, genomic_position_data = set_up_genomic_position_data
+
+        with pytest.raises(ValueError):
+            genomic_position_data.config.viz_backend = "other"
+            genomic_position_data.heatmap(min_values=1)
+        genomic_position_data.config.viz_backend = "plotly"
 
     def test_heatmap_cell_lines(self, set_up_genomic_position_data, mocker):
         _, genomic_position_data = set_up_genomic_position_data
-        mocked_go = mocker.patch("pyllelic.pyllelic.go")
+        mocked_go = mocker.patch("pyllelic.visualization.go")
 
         TEST_CELL_LINE = "test.bam"
         genomic_position_data.heatmap(min_values=1, cell_lines=[TEST_CELL_LINE])
@@ -405,7 +428,7 @@ class Test_GenomicPositionData:
 
     def test_heatmap_modes(self, set_up_genomic_position_data, mocker):
         _, genomic_position_data = set_up_genomic_position_data
-        mocked_go = mocker.patch("pyllelic.pyllelic.go")
+        mocked_go = mocker.patch("pyllelic.visualization.go")
 
         genomic_position_data.heatmap(min_values=1, data_type="modes")
 
@@ -414,43 +437,64 @@ class Test_GenomicPositionData:
 
     def test_heatmap_diffs(self, set_up_genomic_position_data, mocker):
         _, genomic_position_data = set_up_genomic_position_data
-        mocked_go = mocker.patch("pyllelic.pyllelic.go")
+        mocked_go = mocker.patch("pyllelic.visualization.go")
 
         genomic_position_data.heatmap(min_values=1, data_type="diffs")
 
         mocked_go.Figure.assert_called_once()
         mocked_go.Heatmap.assert_called_once()
 
-    def test_heatmap_invalid(self, set_up_genomic_position_data, mocker):
+    def test_heatmap_invalid(self, set_up_genomic_position_data):
         _, genomic_position_data = set_up_genomic_position_data
 
         with pytest.raises(ValueError):
             genomic_position_data.heatmap(min_values=1, data_type="FAKE")
 
+    def test_reads_graph(self, set_up_genomic_position_data, mocker):
+        _, genomic_position_data = set_up_genomic_position_data
+        mocked_px = mocker.patch("pyllelic.visualization.px")
+        mocked_sp = mocker.patch("pyllelic.visualization.sp")
+        mocked_df_plot = mocker.patch("pyllelic.visualization.pd.DataFrame.plot")
+        mocked_mpl = mocker.patch("pyllelic.visualization.plt")
+        mocked_mpl.subplots.return_value = (mocker.MagicMock(), mocker.MagicMock())
+
+        genomic_position_data.reads_graph()
+        genomic_position_data.reads_graph(backend="plotly")
+        genomic_position_data.reads_graph(backend="matplotlib")
+
+        mocked_px.bar.assert_called()
+        mocked_sp.make_subplots.assert_called()
+        mocked_df_plot.assert_called()
+        mocked_mpl.subplots.assert_called()
+
+    def test_reads_graph_cell_lines(self, set_up_genomic_position_data, mocker):
+        _, genomic_position_data = set_up_genomic_position_data
+        mocked_px = mocker.patch("pyllelic.visualization.px")
+        mocked_sp = mocker.patch("pyllelic.visualization.sp")
+        CELL_LINES = ["test"]
+
+        genomic_position_data.reads_graph(cell_lines=CELL_LINES)
+
+        mocked_px.bar.assert_called()
+        mocked_sp.make_subplots.assert_called_once()
+
+    def test_reads_graph_invalid(self, set_up_genomic_position_data):
+        _, genomic_position_data = set_up_genomic_position_data
+
+        with pytest.raises(ValueError):
+            genomic_position_data.reads_graph(backend="FAKE")
+
     def test_summarize_allelelic_data(self, set_up_genomic_position_data):
         _, genomic_position_data = set_up_genomic_position_data
         EXPECTED = pd.DataFrame(
             {
-                "cellLine": {0: "test.bam", 1: "test.bam"},
-                "position": {0: "1294316", 1: "1295770"},
-                "ad_stat": {0: 10.532004907486929, 1: 6.782800511425162},
-                "p_crit": {0: 0.944, 1: 0.957},
-                "diff": {0: 5.551115123125783e-17, 1: -0.052631578947368474},
+                "cellLine": {0: "test"},
+                "position": {0: "1295321"},
+                "ad_stat": {0: 15.372825521573787},
+                "p_crit": {0: 1.009},
+                "diff": {0: -0.024390243902439046},
                 "raw": {
                     0: [
-                        0.3333333333333333,
-                        0.3333333333333333,
-                        0.3333333333333333,
-                        0.3333333333333333,
-                        0.3333333333333333,
-                        0.3333333333333333,
-                        0.3333333333333333,
-                        0.3333333333333333,
-                        0.3333333333333333,
-                        0.3333333333333333,
-                        0.3333333333333333,
-                    ],
-                    1: [
                         0.0,
                         1.0,
                         1.0,
@@ -470,7 +514,29 @@ class Test_GenomicPositionData:
                         1.0,
                         1.0,
                         1.0,
-                    ],
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                        1.0,
+                    ]
                 },
             }
         )
@@ -478,40 +544,39 @@ class Test_GenomicPositionData:
         with np.testing.suppress_warnings() as sup:  # ignore degrees of freedom warning
             sup.filter(RuntimeWarning, "Degrees of freedom")
             sup.filter(module=np.ma.core)
-            actual = genomic_position_data.summarize_allelic_data()
+            CELL_LINES = ["test"]
+            actual1 = genomic_position_data.summarize_allelic_data(
+                cell_lines=CELL_LINES
+            )
+            actual2 = genomic_position_data.summarize_allelic_data()
 
-        pd.testing.assert_frame_equal(EXPECTED, actual)
+        pd.testing.assert_frame_equal(EXPECTED, actual1)
+        pd.testing.assert_frame_equal(EXPECTED, actual2)
 
-    def test__create_methylation_diffs_bar_graph(
-        self, mocker, set_up_genomic_position_data
-    ):
+    def test_sig_methylation_differences(self, set_up_genomic_position_data, mocker):
         _, genomic_position_data = set_up_genomic_position_data
-        mocked_go = mocker.patch("pyllelic.pyllelic.go")
-        intermediate = pd.DataFrame.from_dict(
-            {
-                "cellLine": ["1", "2"],
-                "position": ["1", "2"],
-                "ad_stat": [0, 1],
-                "p_crit": [1, 2],
-                "diff": [1, 2],
-                "raw": [1, 2],
-            }
-        )
-        TEST_DATA = intermediate.astype("object")
+        mocked_go = mocker.patch("pyllelic.visualization.go")
+        mocked_mpl = mocker.patch("pyllelic.visualization.pd.DataFrame.plot")
 
-        _ = genomic_position_data._create_methylation_diffs_bar_graph(TEST_DATA)
+        with np.testing.suppress_warnings() as sup:  # ignore degrees of freedom warning
+            sup.filter(RuntimeWarning, "Degrees of freedom")
+            sup.filter(module=np.ma.core)
+            genomic_position_data.sig_methylation_differences()
+            genomic_position_data.sig_methylation_differences(backend="plotly")
+            genomic_position_data.sig_methylation_differences(backend="matplotlib")
 
-        mocked_go.Figure.assert_called_once()
-        mocked_go.Bar.assert_called_once()
+            mocked_go.Figure.assert_called()
+            mocked_go.Bar.assert_called()
+            mocked_mpl.assert_called_once()
 
-    # def test_sig_methylation_differences(self, set_up_genomic_position_data, mocker):
-    #     _, genomic_position_data = set_up_genomic_position_data
-    #     mocked_go = mocker.patch("pyllelic.pyllelic.go")
+    def test_sig_methylation_differences_invalid(self, set_up_genomic_position_data):
+        _, genomic_position_data = set_up_genomic_position_data
 
-    #     genomic_position_data.sig_methylation_differences()
-
-    #     mocked_go.Figure.assert_called_once()
-    #     mocked_go.Bar.assert_called_once()
+        with np.testing.suppress_warnings() as sup:  # ignore degrees of freedom warning
+            sup.filter(RuntimeWarning, "Degrees of freedom")
+            sup.filter(module=np.ma.core)
+            with pytest.raises(ValueError):
+                genomic_position_data.sig_methylation_differences(backend="FAKE")
 
     def test_anderson_darling_test_with_values(self, set_up_genomic_position_data):
         _, genomic_position_data = set_up_genomic_position_data
@@ -620,73 +685,3 @@ class Test_GenomicPositionData:
         EXPECTED_TRUNCATED_DIFF = TEST_DIFFS.dropna(how="all")
         actual = pyllelic.GenomicPositionData._truncate_diffs(TEST_DIFFS)
         pd.testing.assert_frame_equal(actual, EXPECTED_TRUNCATED_DIFF)
-
-
-# def test_return_individual_data():
-#     """Check whether the expected and result DataFrames are identical."""
-#     in_pos = ["1", "2", "3"]
-#     in_cell = ["TEST1", "TEST2", "TEST3"]
-#     result = pyllelic.return_individual_data(
-#         dict_of_dfs=SAMPLE_DICT_OF_DFS, positions=in_pos, cell_types=in_cell
-#     )
-#     print(result.to_markdown())
-
-#     intermediate = EXPECTED_INTERMEDIATE_INDIVIDUAL_DATA
-
-#     expected = intermediate.astype("object")
-
-#     pd.testing.assert_frame_equal(result, expected)
-
-
-# def test_return_read_values():
-#     """Check whether returned fractional methylation list is correct."""
-#     in_pos = "1"
-#     in_key = "TEST1"
-#     in_min_reads = 1
-#     in_min_sites = 1
-
-#     expected = [
-#         1.0,
-#         1.0,
-#         1.0,
-#         2 / 3,
-#         2 / 3,
-#         2 / 3,
-#     ]
-
-#     result = pyllelic.return_read_values(
-#         pos=in_pos,
-#         key=in_key,
-#         dict_of_dfs=SAMPLE_DICT_OF_DFS,
-#         min_reads=in_min_reads,
-#         min_sites=in_min_sites,
-#     )
-
-#     assert result == expected
-
-
-# def test_get_str_values():
-#     TEST_DATAFRAME = SAMPLE_DICT_OF_DFS.get("TEST1")
-#     TEST_POSITION = "1"
-#     EXPECTED = pd.Series(["111", "111", "111", "011", "011", "011"], name="1")
-#     actual = pyllelic.get_str_values(TEST_DATAFRAME, TEST_POSITION)
-
-#     pd.testing.assert_series_equal(EXPECTED, actual)
-
-
-# def test_find_diffs():
-#     """Check whether the expected and result DataFrames are identical."""
-#     means = EXPECTED_INTERMEDIATE_MEANS
-#     modes = EXPECTED_INTERMEDIATE_MODES
-
-#     expected = EXPECTED_INTERMEDIATE_DIFFS
-
-#     result = pyllelic.find_diffs(means, modes)
-
-#     pd.testing.assert_frame_equal(result, expected)
-
-
-# def test_truncate_diffs():
-#     EXPECTED = EXPECTED_INTERMEDIATE_DIFFS.dropna(how="all")
-#     actual = pyllelic.truncate_diffs(EXPECTED_INTERMEDIATE_DIFFS)
-#     pd.testing.assert_frame_equal(EXPECTED, actual)
